*DECK NXTVCY
      SUBROUTINE NXTVCY(IPRINT,NDIM  ,IDIRC ,MXMESH,MAXSUR,MAXREG,
     >                  MESH  ,DMESH ,NBSUR ,NBREG ,INDXSR,SURVOL)
*
*----------
*
*Purpose:
* Compute the volume and area of each region and surfaces
* for an annular 2-D or 3-D geometry 
* using the NXT tracking procedure.
*
*Copyright:
* Copyright (C) 2005 Ecole Polytechnique de Montreal.
*
*Author(s): 
* G. Marleau.
*
*Update(s): 
* 2005/09/30: Validated with TSTVOL and documented in
*             Report IGE-260.
*
*Reference: 
*  G. Marleau, 
*  \textsl{New Geometries Processing in DRAGON: The NXT: Module},
*  Report IGE-260, \'{E}cole Polytechnique de Montr\'{e}al, 
*  Montr\'{e}al, 2005.
*
*Parameters: input
* IPRINT  print level.
* NDIM    dimension of problem.
* IDIRC   the direction of the first axis of an annular geometry 
*         assuming the axis are in a cyclic rotation.
*         IDIRC      Annulus in 2-D plane Cylinder directions in 3-D  
*         1          (x,y)                z 
*         2          (y,z)                x 
*         3          (z,x)                y
* MXMESH  maximum number of spatial subdivision in
*         $R$ and $X$, $Y$ or $Z$.
* MAXSUR  maximum number of surfaces in the geometry.
* MAXREG  maximum number of regions in the geometry.
* MESH    effective number of spatial subdivision in $R$
*         and $X$, $Y$ or $Z$.
* DMESH   spatial description of the cylinder.
*
*Parameters: output
* NBSUR   number of surfaces in the geometry.
* NBREG   number of regions in the geometry.
* INDXSR  local indexing of surfaces/regions.
* SURVOL  area/volume of regions.
*
*Comments: 
*  1- Contents of the DMESH array:
*     mesh in $X$  is x(i)=DMESH(i,1) for i=0,MESH(1);
*     mesh in $Y$  is y(j)=DMESH(j,2) for j=0,MESH(2);
*     mesh in $Z$  is z(k)=DMESH(k,3) for k=0,MESH(3);
*     if(IDIRC = 1) then
*     ->annular regions in the $X-Y$ plane
*       centre of cylinder in (x,y)=(DMESH(-1,1),DMESH(-1,2))
*       radius of shells      r(l)=DMESH(l,4), l=1,MESH(4)
*     else if(IDIRC = 2) then
*     ->annular regions in the $Y-Z$ plane
*       centre of cylinder in (y,z)=(DMESH(-1,2),DMESH(-1,3))
*       radius of shells      r(l)=DMESH(l,4), l=1,MESH(4)
*     else if(IDIRC = 3) then
*     ->annular regions in the $Z-X$ plane
*       centre of cylinder in (z,x)=(DMESH(-1,3),DMESH(-1,1))
*       radius of shells      r(l)=DMESH(l,4), l=1,MESH(4)
*     endif
*  2- Contents of the INDXSR array:
*     For i>0
*       INDXSR(1,i)= ix is the $X$ location of region i 
*       INDXSR(2,i)= iy is the $Y$ location of region i 
*       INDXSR(3,i)= iz is the $Z$ location of region i
*       INDXSR(4,i)= ir =0 is the $R$ location of region i.
*     For i<0
*       INDXSR(1,i)= ix is the $X$ location of surface i 
*       INDXSR(2,i)= iy is the $Y$ location of surface i 
*       INDXSR(3,i)= iz is the $Z$ location of surface i
*       INDXSR(4,i)= ir =0 is the $R$ location of surface i.
*       with INDXSR(n,i)=-1 for surface associated with 
*                           location 0 in direction n.
*       with INDXSR(n,i)=-2 for surface associated with 
*                           location MESH(n) in direction n.
*       Note that for radial regions INDXSR(n,i)=-1 does not
*       exists.
*
*----------
*
      IMPLICIT         NONE
*----
*  Subroutine arguments
*----
      INTEGER          IPRINT,NDIM,IDIRC,MXMESH,MAXSUR,MAXREG
      INTEGER          MESH(4)
      DOUBLE PRECISION DMESH(-1:MXMESH,4)
      INTEGER          NBSUR,NBREG,INDXSR(4,-MAXSUR:MAXREG)
      DOUBLE PRECISION SURVOL(-MAXSUR:MAXREG)
*----
*  Local parameters
*----
      INTEGER          IOUT
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NAMSBR='NXTVCY')
      INTEGER          MAXDIM
      PARAMETER       (MAXDIM=4)
      DOUBLE PRECISION DZERO,DONE,DTWO
      PARAMETER       (DZERO=0.0D0,DONE=1.0D0,DTWO=2.0D0)
*----
*  Functions
*----
      DOUBLE PRECISION XDRCST,PI
*----
*  Local variables
*----
      INTEGER          IDIR(MAXDIM),NM(MAXDIM),IDM(MAXDIM)
      INTEGER          IDGP1,IDGP2,IDGPP,IR,IA,IVOL,ISUR,ISUR2
      DOUBLE PRECISION DA,DSR,R2I,R2O,CIRC
*----
*  Data
*----
      CHARACTER        CDIR(MAXDIM)*1
      SAVE             CDIR
      DATA             CDIR /'X','Y','Z','R'/
*----
*  Prepare radial and axial loops 
*  as a function of IDIRC and NDIM.
*  NM(4) is radial part
*  NM(2) is axial part (in 3-d)
*----
      PI=XDRCST('Pi',' ')
      CALL XDISET(INDXSR,4*(MAXSUR+MAXREG+1),0)
      CALL XDDSET(SURVOL,(MAXSUR+MAXREG+1),DZERO)
      IF(NDIM .EQ. 1) CALL XABORT(NAMSBR//
     >': Only 2-D and 3-D problems permitted')
      IDGP1=IDIRC
      IDGP2=MOD(IDIRC,3)+1
      IDGPP=MOD(IDIRC+1,3)+1
      IDIR(1)=IDGP1
      IDIR(2)=IDGP2
      IDIR(3)=IDGPP
      IDIR(4)=4
      NM(4)=MESH(4)
      IDM(4)=1
      NM(3)=1
      IDM(3)=0
      IF(NDIM .EQ. 3) THEN
        NM(3)=MESH(IDGPP)
        NBSUR=NM(3)+2*NM(4)
        NBREG=NM(4)*NM(3)
        IDM(3)=1
      ELSE
        NBSUR=1
        NBREG=NM(4)
      ENDIF
      IF(MAXREG .LT. NBREG) CALL XABORT(NAMSBR//
     >': Insufficient space to store region volumes')
      IF(MAXSUR .LT. NBSUR) CALL XABORT(NAMSBR//
     >': Insufficient space to store surface areas')
*----
*  Print mesh if required
*----
      IF(IPRINT .GE. 100) THEN
        WRITE(IOUT,6000) NAMSBR
        WRITE(IOUT,6002) 'CENTER'//CDIR(IDGP1)//CDIR(IDGP2)
        WRITE(IOUT,6006) (DMESH(-1,IDIR(IA)),IA=1,2)
        WRITE(IOUT,6003) 
        WRITE(IOUT,6002) 'RADIAL'//CDIR(IDGP1)//CDIR(IDGP2)
        WRITE(IOUT,6006) (DMESH(IA,4),IA=1,MESH(4))
        WRITE(IOUT,6003) 
        IF(NDIM .EQ. 3) THEN
          WRITE(IOUT,6002) 'AXIALD'//CDIR(IDGPP)
          WRITE(IOUT,6006) (DMESH(IA,IDGPP),IA=0,MESH(IDGPP))
          WRITE(IOUT,6003) 
        ENDIF
      ENDIF
*----
*  1- Top and bottom axial surfaces
*----
      ISUR=0
      IF(NDIM .EQ. 3) THEN
        ISUR2=ISUR+NM(4)
        R2I=DZERO
        DO 111 IR=1,NM(4)
          R2O=DMESH(IR,4)*DMESH(IR,4)
          DSR=PI*(R2O-R2I)
          R2I=R2O
          ISUR=ISUR+1
          SURVOL(-ISUR)=DSR
          INDXSR(IDGP1,-ISUR)=0
          INDXSR(IDGP2,-ISUR)=0
          INDXSR(IDGPP,-ISUR)=-1
          INDXSR(4,-ISUR)=IR
          ISUR2=ISUR2+1
          SURVOL(-ISUR2)=DSR
          INDXSR(IDGP1,-ISUR2)=0
          INDXSR(IDGP2,-ISUR2)=0
          INDXSR(IDGPP,-ISUR2)=-2
          INDXSR(4,-ISUR2)=IR
 111    CONTINUE
        ISUR=ISUR2
      ENDIF
*----
*  Compute surface area
*  2- Radial surfaces
*----
      CIRC=DTWO*PI*DMESH(NM(4),4)
      DO 110 IA=1,NM(3)
        DA=DONE
        IF(IDM(3) .EQ. 1) 
     >  DA=DMESH(IA,IDGPP)-DMESH(IA-1,IDGPP)
        ISUR=ISUR+1
        SURVOL(-ISUR)=DA*CIRC
        INDXSR(IDGP1,-ISUR)=0
        INDXSR(IDGP2,-ISUR)=0
        INDXSR(IDGPP,-ISUR)=IA
        INDXSR(4,-ISUR)=-2
 110  CONTINUE
*----
*  Computes regional volumes
*  Loop on axial ($X$, $Y$ or $Z$) directions (130) 
*----
      IVOL=0
      DO 120 IA=1,NM(3)
        DA=DONE
        IF(IDM(3) .EQ. 1) 
     >  DA=DMESH(IA,IDGPP)-DMESH(IA-1,IDGPP)
*----
*  Loop on radial $R$ directions (140)
*----
        R2I=DZERO
        DO 121 IR=1,NM(4)
          R2O=DMESH(IR,4)*DMESH(IR,4)
          DSR=PI*(R2O-R2I)
          R2I=R2O
          IVOL=IVOL+1
          SURVOL(IVOL)=DSR*DA
          INDXSR(IDGP1,IVOL)=0
          INDXSR(IDGP2,IVOL)=0
          INDXSR(IDGPP,IVOL)=IA
          INDXSR(4,IVOL)=IR
 121    CONTINUE
 120  CONTINUE
      IF(IPRINT .GE. 100) THEN
        WRITE(IOUT,6002) 'SurVol'
        WRITE(IOUT,6005) (IVOL,(INDXSR(IR,IVOL),IR=1,4),SURVOL(IVOL),
     >                     IVOL=-NBSUR,NBREG)
        WRITE(IOUT,6003) 
        WRITE(IOUT,6001) NAMSBR
      ENDIF
      RETURN
*----
*  Output formats
*----
 6000 FORMAT('(* Output from --',A6,'-- follows ')
 6001 FORMAT('   Output from --',A6,'-- completed *)')
 6002 FORMAT(A12,'={')
 6003 FORMAT('};')
 6005 FORMAT((5(I10,','),F20.10,:,','))
 6006 FORMAT(4(F20.10,:,','))
*----
*  Warning formats
*----
 9000 FORMAT(1X,'Warning from --',A6,'--'/
     >       1X,'Number of surface found is ',I5/
     >       1X,'Only 0 or 2 expected')
      END 
